import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;

import java.io.File;
import java.io.FileOutputStream;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import javax.swing.JButton;
import java.io.IOException;
import java.util.Scanner;
import java.io.*;

import java.lang.InterruptedException;

class Capture implements Runnable{
	File file;
	Record record;
	AudioInputStream audioInputStream;
	String errStr;
	TargetDataLine line;
	Thread thread;
	double duration;
	int bufSize = 16384;
	String filename;
	double recordtime;

	public Capture(Record rc, AudioInputStream ais, double time){
		record = rc;
		audioInputStream = ais;
		recordtime = time;
	}

	public void start() {
		record.recording = true;
		errStr = null;
		thread = new Thread(this);
		thread.setName("Capture");
		System.out.println("Thread started");
		thread.start();
		try{
			thread.join((int) recordtime);
			System.out.println("2 Seconds?");
			System.out.println("Duration: " + duration);
			record.capture.stop();
		} catch (InterruptedException ex) {
			ex.printStackTrace();
		}
	}

	public void stop() {
		thread = null;
	}

	private void shutDown(String message) {
		if ((errStr = message) != null && thread != null) {
			thread = null;
			System.err.println(errStr);
		}
	}

	public void run() {
		duration = 0;
		audioInputStream = null;
		AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
		float rate = 8000.0f;
		int channels = 2;
		int frameSize = 4;
		int sampleSize = 16;
		boolean bigEndian = true;

    	AudioFormat format = new AudioFormat(encoding, rate, sampleSize, channels, (sampleSize / 8)
        * channels, rate, bigEndian);

		DataLine.Info info = new DataLine.Info(TargetDataLine.class, format);

		if (!AudioSystem.isLineSupported(info)) {
			shutDown("Line matching " + info + " not supported.");
			return;
		}

		try {
			line = (TargetDataLine) AudioSystem.getLine(info);
			line.open(format, line.getBufferSize());
		} catch (LineUnavailableException ex) {
			shutDown("Unable to open the line: " + ex);
			return;
		} catch (SecurityException ex) {
			shutDown(ex.toString());
			//JavaSound.showInfoDialog();
			return;
		} catch (Exception ex) {
			shutDown(ex.toString());
			return;
		}

    	// play back the captured audio data
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		int frameSizeInBytes = format.getFrameSize();
		int bufferLengthInFrames = line.getBufferSize() / 8;
		int bufferLengthInBytes = bufferLengthInFrames * frameSizeInBytes;
		byte[] data = new byte[bufferLengthInBytes];
		int numBytesRead;

    	line.start();

		while (thread != null) {
      		if ((numBytesRead = line.read(data, 0, bufferLengthInBytes)) == -1) {
        		break;
      		}
      		out.write(data, 0, numBytesRead);
    	}

		// we reached the end of the stream.
		// stop and close the line.
		line.stop();
		line.close();
		line = null;

		// stop and close the output stream
		try {
		  out.flush();
		  out.close();
		} catch (IOException ex) {
		  ex.printStackTrace();
		}

		// load bytes into the audio input stream for playback

		byte audioBytes[] = out.toByteArray();
		ByteArrayInputStream bais = new ByteArrayInputStream(audioBytes);
		audioInputStream = new AudioInputStream(bais, format, audioBytes.length / frameSizeInBytes);

    	long milliseconds = (long) ((audioInputStream.getFrameLength() * 1000) / format.getFrameRate());

    	duration = milliseconds / 1000.0;
		System.out.println("TERMINA DE GRABAR");
		System.out.println("Duracion: " + duration);

		FileOutputStream fop = null;
		File file;

		try {
			file = new File("s.txt");
			fop = new FileOutputStream(file);

			// if file doesnt exists, then create it
			if (!file.exists()) {
			file.createNewFile();
			}
			String espacio = "\n";

			// get the content in bytes
			//byte[] contentInBytes = content.getBytes();
			byte[] b = new byte[4];
			for (int i = 0; i < (int)(audioBytes.length/4); i++) {
				for(int j = 0; j<4;j++){
					b[0] = audioBytes[4*i];
					b[1] = audioBytes[4*i + 1];
					b[2] = audioBytes[4*i + 2];
					b[3] = audioBytes[4*i + 3];					
				}
				fop.write((Integer.toString(byteArrayToInt(b))).getBytes());
				fop.write(espacio.getBytes());
			}
			fop.flush();
			fop.close();
			record.recording = false;
			System.out.println("Recording: " + record.recording);
			record.checkAnswer();
		} catch (IOException e) {
			e.printStackTrace();
      		return;
    	}

    	try {
			audioInputStream.reset();
		} catch (Exception ex) {
			ex.printStackTrace();
			return;
		}
	}

	public static int byteArrayToInt(byte[] b) {
		  int myInt = (b[0] << 24) | ((b[1] & 0xff) << 16) | ((b[2] & 0xff) << 8) | (b[3] & 0xff);
		  return myInt;
	}

}
